@@ -1,18 +1,24 @@
+"""
+Module containg all the main methods of the package. 
+"""
+
+
 import io
-import numpy as np
 import pkgutil
 import re
-import wikipediaapi
-
-from .foodcategory import CategorySynset
-from .foodcategory import FoodCategory
 from collections import Counter
+
+import numpy as np
+import wikipediaapi
 from nltk.corpus import wordnet
 from PyDictionary import PyDictionary
 from pyfood.utils import Shelf
 from textblob import Word
 
-embedding_path = 'data/ingredient_embedding.npy'
+from .foodcategory import CategorySynset, FoodCategory
+
+embedding_path = "data/ingredient_embedding.npy"
+
 
 def __get_embedding():
     """
@@ -23,18 +29,20 @@ def __get_embedding():
     embedding_io = io.BytesIO(pkgutil.get_data(__name__, embedding_path))
     return np.load(embedding_io, allow_pickle=True).item()
 
+
 def lemmatize_word(word):
     """
     Lemmatize the provided word.
-    Lemmatization is the process of converting a word to its base form. 
+    Lemmatization is the process of converting a word to its base form.
 
     :param word: the word to be lemmatized.
-    :return: the word lemmatized. 
+    :return: the word lemmatized.
     """
     w = Word(word)
     return w.lemmatize()
 
-def is_ingredient_vegan(ingredient): 
+
+def is_ingredient_vegan(ingredient):
     """
     Check if the provided ingredient is vegan or not.
 
@@ -42,9 +50,10 @@ def is_ingredient_vegan(ingredient):
     :return: a bool indicating whether the ingredient is vegan or not.
     """
     ingredient = ingredient.strip()
-    shelf = Shelf('Milan', month_id=0)
+    shelf = Shelf("Milan", month_id=0)
     results = shelf.process_ingredients([ingredient])
-    return results['labels']['vegan']
+    return results["labels"]["vegan"]
+
 
 def is_recipe_vegan(ingredients):
     """
@@ -52,43 +61,50 @@ def is_recipe_vegan(ingredients):
     If only one element is not vegan the recipe is not vegan.
 
     :param ingredients: the list of the ingredients.
-    :return: a bool indicating wheter the recipe is vegan or not. 
+    :return: a bool indicating wheter the recipe is vegan or not.
     """
-    shelf = Shelf('Milan', month_id=0)
-    results=shelf.process_ingredients(ingredients)
-    return results['labels']['vegan']
+    shelf = Shelf("Milan", month_id=0)
+    results = shelf.process_ingredients(ingredients)
+    return results["labels"]["vegan"]
+
 
 def add_ingredient(ingredient, tag):
     """
-    Map the provided ingredient and the tag into the embedding dataset. 
-    Tag must be one the following FoodCategory: 
-    vegetable, fruit, meat, legume, diary, egg, staple, condiment, nut, seafood, dessert
+    Map the provided ingredient and the tag into the embedding dataset.
+    Tag must be one the following FoodCategory:
+    vegetable, fruit, meat, legume, diary, egg, staple,
+    condiment, nut, seafood, dessert
 
     :param ingredient: the name of the ingredient.
     :param tag: the class of the ingredient. Must be one of the listed above.
-    :return: a bool indicating if the operation has succeded or not. 
+    :return: a bool indicating if the operation has succeded or not.
     """
     embedding = __get_embedding()
     ingredient = ingredient.strip()
     tag = tag.strip()
     if ingredient in embedding:
         return False
-    
+
     embedding[ingredient] = FoodCategory[tag].value
     return True
 
+
 def search_ingredient_hypernyms(ingredient):
     """
-    Predict the class of the provided ingredient based on the Wu & Palmer’s similarity between
-    ingredient, his hypernyms and the 11 FoodCategory. 
-    The FoodCategory is choosen based on the maximum similarity value between the ingredient, 
-    its hypernym and the various categories. If the predicted category is different between ingredient
-    and hypernym the category is choosen based on the avarege of both. 
+    Predict the class of the provided ingredient based on the Wu & Palmer’s
+    similarity between ingredient, his hypernyms and the 11 FoodCategory.
+    The FoodCategory is choosen based on the maximum similarity value between
+    the ingredient, its hypernym and the various categories. If the predicted
+    category is different between ingredient and hypernym the category is
+    choosen based on the avarege of both.
 
     :param ingredient: the name of the ingredient.
     :return: the class of the ingredient.
     """
-    ingredient = wordnet.synset(f'{ingredient}.n.01')
+    if " " in ingredient:
+        ingredient = ingredient.split(" ")[-1]
+
+    ingredient = wordnet.synsets(ingredient)[0]
     hypernym = ingredient.hypernyms()[0]
     categories = CategorySynset.categories
 
@@ -97,42 +113,51 @@ def search_ingredient_hypernyms(ingredient):
     for cat in categories:
         sim.append(ingredient.wup_similarity(cat))
         hypernym_sim.append(hypernym.wup_similarity(cat))
-    
+
     best_sim = sim.index(max(sim))
     best_hyp = hypernym_sim.index(max(hypernym_sim))
 
-    if best_sim == best_hyp: 
+    if best_sim == best_hyp:
         return FoodCategory(best_sim).name
-    else: 
-        sum = [(x + y)/2 for x, y in zip(sim, hypernym_sim)]
+    else:
+        sum = [(x + y) / 2 for x, y in zip(sim, hypernym_sim)]
         return FoodCategory(sum.index(max(sum)))
 
+
 def search_ingredient_class(ingredient):
     """
-    Search on wikipedia and english dictionary the class of the provided ingredient.
-    Returns the most occurrences of a single FoodCategory class based on the two research. 
+    Search on wikipedia and english dictionary the class of
+    the provided ingredient.
+    Returns the most occurrences of a single FoodCategory class based on
+    the two research.
 
     :param ingredient: the name of the ingredient.
     :return: the class of the ingredient.
     """
+    if " " in ingredient:
+        ingredient = ingredient.split(" ")[-1]
+
     dictionary = PyDictionary()
-    wiki = wikipediaapi.Wikipedia('en')
+    wiki = wikipediaapi.Wikipedia("en")
 
     page = wiki.page(ingredient)
-    ontology = ', '.join(dictionary.meaning(ingredient)['Noun'])
-    
+    meaning = dictionary.meaning(ingredient)["Noun"]
+    ontology = ", ".join(meaning) if meaning else ""
+
     categories = []
     for category in FoodCategory:
-        if page and re.search(r'\b({0})\b'.format(category.name), page.summary):
+        if page and re.search(r"\b({0})\b".format(category.name), page.summary):
             categories.append(category.name)
-        if re.search(r'\b({0})\b'.format(category.name), ontology):
+        if ontology and re.search(r"\b({0})\b".format(category.name), ontology):
             categories.append(category.name)
     return max(categories, key=categories.count) if len(categories) else None
 
+
 def get_ingredient_class(ingredient):
     """
-    Predict the class of the provided ingredient based on the embeddings. 
-    If the ingredient cannot be found in the dictionary it will be searched on wikipedia pages or hypernyms.
+    Predict the class of the provided ingredient based on the embeddings.
+    If the ingredient cannot be found in the dictionary it will be
+    searched on wikipedia pages or hypernyms.
 
     :param ingredient: the name of the ingredient.
     :return: the class of the ingredient.
@@ -147,26 +172,30 @@ def get_ingredient_class(ingredient):
         hyp_class = search_ingredient_hypernyms(lemmatized_ing)
         return web_class if web_class else hyp_class
 
+
 def get_recipe_class_percentage(ingredients):
     """
-    Classify a recipe in tags based on its ingredient. 
+    Classify a recipe in tags based on its ingredient.
     Returns the percentages of ingredient class in the recipe provided.
 
     :param ingredients: list of ingredients in the recipe.
-    :return: list of tuples containg classes and percentages. 
+    :return: list of tuples containg classes and percentages.
     """
-    tags = [get_ingredient_class(ingredient) for ingredient in ingredients]  
+    tags = [get_ingredient_class(ingredient) for ingredient in ingredients]
     c = Counter(tags)
-    return [(i, str(round(c[i] / len(tags) * 100.0, 2)) + '%') for i in c]
+    return [(i, str(round(c[i] / len(tags) * 100.0, 2)) + "%") for i in c]
+
 
 def get_recipe_tags(ingredients):
     """
-    Classify a recipe in tags based on its ingredient. 
-    Tag could be: Vegetable, Fruit, Meat, Legume, Diary, Egg, Staple, Condiment, Nut, Seafood 
+    Classify a recipe in tags based on its ingredient.
+    Tag could be: Vegetable, Fruit, Meat, Legume, Diary,
+    Egg, Staple, Condiment, Nut, Seafood
 
     :param ingredients: list of ingredients in the recipe.
-    :return: set of tags for the recipe. 
+    :return: set of tags for the recipe.
     """
     tags = [get_ingredient_class(ingredient) for ingredient in ingredients]
-    if None in tags: tags.remove(None)
-    return list(set(tags)) if len(tags) else tags    
\ No newline at end of file
+    if None in tags:
+        tags.remove(None)
+    return list(set(tags)) if len(tags) else tags
